/**
 * Programa para crear una bolsa. 
 * @author Yazmin Gonzalez Bayardo
 * @version Febrero 2015
 */
public class Bolsa implements InterfazBolsa {
	
	protected Object[] bolsa; //Esta implementacion de Bolsa usara una arreglo fijo
	protected int numElem;
	
	/**
	 * Constructor por omision que crea una bolsa para 15 elementos
	 */
	public Bolsa() {
		this(15);
	}
	
	/**
	 * Constructor de una bolsa que recibe el numero de elementos indicados
	 * @param tam - capacidad de elementos de la bolsa
	 */
	public Bolsa(int tam) {
		bolsa = new Object[tam>0? tam : 15];
		numElem = 0;
	}
	
	/**
	 * Constructor de copia
	 * @param b - Bolsa que se copiara para crear una nueva Bolsa
	 */
	public Bolsa(Bolsa b) {
		bolsa = new Object[b.bolsa.length];
		
		for(int i=0; i < b.numElem; i++) {
			bolsa[i] = b.bolsa[i];
		}
		numElem = b.numElem;
	}
	
    /**
     * Determina el numero de elementos en la bolsa
     * @return int - Cantidad de elementos en la coleccion
     */
	public int numElementos() {
		return numElem;
	}
	
    /**
     * Agrega un elemento a la bolsa si hay espacio disponible 
     * @param elemento - Elemento que se incluye a la bolsa
	 * @return boolean - Devuelve true si la operacion fue exitosa o false en otro caso
     */
	public boolean agregar(Object elemento) {
		if(numElem >= bolsa.length)
			return false;
				
		bolsa[numElem] = elemento;
		numElem++;
		return true;
	}
	
    /**
     * Elimina de la bolsa el elemento indicado todas las veces que aparece en la coleccion
     * @param elemento - Elemento que se elimina de la bolsa
	 * @return boolean - Devuelve true si la operacion fue exitosa o false en otro caso
     */
	public boolean eliminar(Object elemento) {
		boolean existe = false;
		for(int i = 0; i < numElem; i++) {
			if(bolsa[i].equals(elemento)) {
				bolsa[i] = bolsa[numElem-1]; // los elementos de una bolsa no estan ordenados, por lo que basta con sobre-escribir el elemento a eliminar con el elemento en la ultima posicion del arreglo
				i--;
				numElem--;
				existe = true;
			}
		}
		return existe; //No se encontro el elemento que sea deseaba eliminar
	}
		
    /**
     * Determina si la bolsa contiene al menos una vez al elemento indicado
     * @param elemento - Elemento que se va a buscar en la bolsa
     * @return boolean - Devuelve true si el elemento esta en la coleccion y false en otro caso
     */
	public boolean contiene (Object elemento) {
		for(int i = 0; i < numElem; i++) {
			if(bolsa[i].equals(elemento))
				return true;
		}
		return false;
	}
	
	/**
	 * Determina el numero de veces que un elemento indicado se repite en la coleccion
	 * @param elemento - Elemento que se desea saber cuantas veces aparece en la bolsa
	 * @return int - Devuelve el numero de veces que el elemento aparece en la bolsa	
	 */
	public int cardinalidad (Object elemento) {
		int c = 0;
		for(int i = 0; i < numElem; i++) {
			if(bolsa[i].equals(elemento))
				c++;
		}
		return c;
	}
}