/**
 * Clase para trabajar con bombas de gasolina.
 * @author Yazmin Gonzalez Bayardo
 * @version Junio 2015
 */
public class BombaGasolina {
	
	private static int cont = 0;
	private final int id;
	private int gasDisponible;
	private int distanciaSgte;
	
	/**
	 * Constructor de una bomba de gasolina.
	 * @param gasDisponible -- cantidad de litros de gasolina disponibles.
	 * @param distanciaSgte -- cantidad de kilometros de distancia para llegar a la siguiente bomba de gasolina.
	 */
	public BombaGasolina(int gasDisponible, int distanciaSgte) {
		id = ++cont;
		this.gasDisponible = gasDisponible;
		this.distanciaSgte = distanciaSgte;
	}
	
	/**
	 * Metodo para obtener el identificador de la bomba de gasolina.
	 * @return int -- identificador de la bomba de gasolina.
	 */
	public int obtenerId() {
		return id;
	}
	
	/**
	 * Metodo para obtener la cantidad de gasolina disponible de la bomba.
	 * @return int -- cantidad de gasolina que puede proporcionar la bomba de gasolina.
	 */
	public int obtenerGasDisponible() {
		return gasDisponible;
	}
	
	/**
	 * Metodo para obtener la distancia que hay para llegar a la siguiente bomba de gasolina.
	 * @return int -- distancia para llegar a la siguiente bomba de gasolina.
	 */
	public int obtenerDistanciaSgte() {
		return distanciaSgte;
	}
	
	/**
	 * Metodo para asignar la cantidad de gasolina disponible de la bomba.
	 * @param gas -- cantidad de litros de gasolina que tendra disponible la bomba de gasolina.
	 */
	public void asignarGasDisponible(int gas) {
		gasDisponible = gas;
	} 
	
	/**
	 * Metodo para asignar la distancia para llegar a la siguiente bomba de gasolina.
	 * @param dist -- kilometros de distancia para llegar a la siguiente bomba de gasolina.
	 */
	public void asignarDistanciaSgte(int dist) {
		distanciaSgte = dist;
	}
	
	/**
	 * Metodo para determinar si bombas de gasolina son iguales.
	 * @param o -- bomba de gasolina contra la cual se va a comparar.
	 * @return boolean -- true si son iguales y false en otro caso.
	 */
	public boolean equals(Object o) {
		if (o == this) return true;
		if (o == null) return false;
		if (o instanceof BombaGasolina) {
			BombaGasolina bg = (BombaGasolina)o;
			return obtenerId() == bg.obtenerId();
		}
		return false;
	}
	
	/**
	 * Metodo para convertir una bomba de gasolina en una cadena de caracteres.
	 * @return String - la bomba de gasolina en formato de cadena.
	 */
	public String toString() {
		return "\nBomba # " + id + "   Gasolina disponible: " + gasDisponible + "   Distancia siguiente bomba: " + distanciaSgte;
	}
}