/**
 * Clase para ejemplificar definiciones de arboles binarios y realizar recorridos sobre ellos.
 * @author Yazmin Gonzalez Bayardo
 * @version Julio 2015
 */
public class ArbolBinario {
	
	private NodoArbol raiz;
	
	/**
	 * Constructor por omision.
	 */
	public ArbolBinario() {
		raiz = null;
	}
	
	/**
	 * Constructor que inicializa un arbol binario con el nodo raiz indicado como parametro.
	 * @param raiz -- nodo raiz con el que se inicializa el arbol binario.
	 */
	public ArbolBinario(NodoArbol raiz) {
		this.raiz = raiz;
	}
	
	/**
	 * Metodo para insertar un elemento al arbol binario.
	 * @param elemento -- dato a insertar
	 */
	public void insertar(Object elemento) {
		raiz = insertar(elemento,raiz);
	}
	
    /**
     * Metodo interno para insertar un elemento al arbol binario. Se busca mantener balanceado el arbol consultando el tamanio
	 * de los subarboles.
     * @param elemento -- dato a insertar
     * @param nodo -- la raiz del arbol
     * @return NodoArbol -- la nueva raiz
     */
	private NodoArbol insertar(Object elemento, NodoArbol nodo) {
		if( nodo == null) {
			nodo = new NodoArbol(elemento);
		}
		else {
			if(tamanio(nodo.izquierda) <= tamanio(nodo.derecha))
				nodo.izquierda = insertar(elemento,nodo.izquierda);
			else
				nodo.derecha = insertar(elemento,nodo.derecha);
		}
		return nodo;
	}
	
	/**
	 * Determina la cantidad de nodos en el arbol binario.
	 * @return int -- numero de nodos del arbol
	 */
	public int tamanio() {
		return tamanio(raiz);
	}
	
	/**
	 * Metodo interno para determinar el numero de nodos de un arbol binario a partir del nodo dado como parametro.
	 * @param nodo -- nodo a partir del cual se calcula su tamanio
	 @ @return int -- numero de nodos del arbol binario
	 */
	private int tamanio(NodoArbol nodo) {
		if(nodo == null)
			return 0;
		else
			return 1 + tamanio(nodo.izquierda) + tamanio(nodo.derecha);
	}
	
	/**
	 * Determina la altura del arbol binario. Es el numero maximo de niveles que tiene.
	 * @return int -- altura del arbol binario
	 */
	public int altura() {
		return altura(raiz);
	}
	
	/**
	 * Metodo interno para calcular la altura de un arbol binario a partir de la raiz indicada como parametro.
	 * @param nodo -- el nodo raiz del arbol binario
	 * @return int -- altura del arbol binario
	 */
	private int altura(NodoArbol nodo) {
		if(nodo == null)
			return 0;
		else
			return 1 + java.lang.Math.max(altura(nodo.izquierda),altura(nodo.derecha));
	}
	
	/** 
	 * Determina el numero de hojas en el arbol binario.
	 * @return int -- cantidad de hojas del arbol binario
	 */
	public int hojas() {
		return hojas(raiz);
	}
	
	/**
	 * Metodo interno para determinar el numero de hojas en un arbol binario.
	 * @param nodo -- nodo raiz del arbol binario a partir del cual se calculara su numero de hojas
	 * @return int -- cantidad de hojas del arbol binario
	 */
	private int hojas(NodoArbol nodo) {
		if(nodo==null)
			return 0;
		else
		{
			if(nodo.izquierda == null && nodo.derecha == null)
				return 1;
			else
				return hojas(nodo.izquierda) + hojas(nodo.derecha);
					
		}
	}
	
	/** 
	 * Determina el numero de nodos internos en el arbol binario.
	 * @return int -- cantidad de nodos internos del arbol binario
	 */
	public int nodosInternos() {
		return nodosInternos(raiz) - 1;
	}
	
	/**
	 * Metodo privado para determinar el numero de nodos internos en un arbol binario.
	 * @param nodo -- nodo raiz del arbol binario a partir del cual se calculara su numero de nodos internos
	 * @return int -- cantidad de nodos internos del arbol binario
	 */
	private int nodosInternos(NodoArbol nodo) {
		if(nodo==null)
			return 0;
		else
		{
			if(nodo.izquierda != null || nodo.derecha != null)
				return 1 + nodosInternos(nodo.izquierda) + nodosInternos(nodo.derecha);
			else
				return 0;
		}
	}
	
	/**
	 * Imprime el valor contenido en el nodo indicado como parametro.
	 * @param nodo -- nodo cuyo valor se imprimira en pantalla
	 */
	public void imprimir(NodoArbol nodo) {
		if(nodo != null)
			System.out.print(nodo.valor + " ");
	}
	
	
	/**
	 * Recorrido del arbol binario en pre-Orden.
	 */
	public void preOrden() {
		preOrden(raiz);
	}
	
	/**
	 * Metodo interno para recorrido en pre-Orden a partir del nodo indicado como parametro.
	 * @param nodo -- nodo a partir del cual se realiza el recorrido
	 */
	private void preOrden(NodoArbol nodo) {
		if(nodo != null) {
			imprimir(nodo);
			preOrden(nodo.izquierda);
			preOrden(nodo.derecha);
		}
	}
	
	/**
	 * Recorrido del arbol binario en in-Orden.
	 */
	public void inOrden() {
		inOrden(raiz);
	}
	
	/**
	 * Metodo interno para recorrido en in-Orden a partir del nodo indicado como parametro.
	 * @param nodo -- nodo a partir del cual se realiza el recorrido
	 */
	private void inOrden(NodoArbol nodo) {
		if(nodo != null) {
			inOrden(nodo.izquierda);
			imprimir(nodo);
			inOrden(nodo.derecha);
		}
	}
	
	/**
	 * Recorrido del arbol binario en post-Orden.
	 */
	public void postOrden() {
		postOrden(raiz);
	}
	
	/**
	 * Metodo interno para recorrido en post-Orden a partir del nodo indicado como parametro.
	 * @param nodo -- nodo a partir del cual se realiza el recorrido
	 */
	private void postOrden(NodoArbol nodo) {
		if(nodo != null) {
			postOrden(nodo.izquierda);
			postOrden(nodo.derecha);
			imprimir(nodo);
		}
	}
}