/**
 * Clase para decodificar un mensaje en codigo Morse.
 * @author Yazmin Gonzalez Bayardo
 * @version Julio 2015
 */
public class ArbolMorse {
	
	// Variable estatica para la raiz del arbol binario. Se asigna una cadena vacia en el nodo raiz.
	private static NodoArbol raiz = new NodoArbol("");
	
	/**
	 * Bloque estatico que inicializa el arbol binario con codigo Morse. 
	 */
	static {
		crearArbolMorse();
	}
	
	/**
	 * Metodo interno para construir el arbol binario con el codigo Morse.
	 */
	private static void crearArbolMorse() {
		String[] letras = {"A","B","C","D","E","F","G","H","I","J","K",
			"L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"};
		String[] codigos = {".-","-...","-.-.","-..",".","..-.","--.","....","..",".---","-.-",
			".-..","--","-.","---",".--.","--.-",".-.","...","-","..-","...-",".--","-..-","-.--","--.."};
		
		for(int i=0; i<letras.length; i++){
			raiz = insertarCodigo(letras[i],codigos[i],raiz);
		}
	}
	
	/**
	 * Metodo interno para insertar un elemento al arbol de codigo Morse.
	 * @param letra -- la letra del alfabeto que se desea agregar.
	 * @param codigo -- el codigo Morse correspondiente a la letra del alfabeto.
	 * @param nodo -- la raiz del arbol con codigo Morse.
	 * @return NodoArbol -- la nueva raiz del arbol con codigo Morse.
	 */
	private static NodoArbol insertarCodigo(String letra, String codigo, NodoArbol nodo) {
		
		if(codigo.length() == 0)
		{
			nodo.valor=letra;
		}
		else{
			char c = codigo.charAt(0);
			codigo = codigo.substring(1);
			if(c == '.'){
				if(nodo.izquierda == null)
					nodo.izquierda = new NodoArbol("");
				nodo.izquierda = insertarCodigo(letra, codigo, nodo.izquierda);
			}	
			else{
				if(nodo.derecha == null)
					nodo.derecha = new NodoArbol("");
				nodo.derecha = insertarCodigo(letra, codigo, nodo.derecha);
			}
		}
		return nodo;
	}
	
	/**
	 * Metodo para decodificar un mensaje en codigo Morse.
	 * Muestra en pantalla tanto el mensaje en codigo Morse como el mensaje decodificado.
	 * @param msjMorse -- mensaje que se desea decodificar
	 */
	public static void decodificar(String msjMorse) {
		String[] palabras = msjMorse.split("  ");
		StringBuilder sb = new StringBuilder();
		
		for(String p : palabras) {
			
			String[] codigos = p.split(" "); 
			for(String c : codigos) {
				sb.append(decodCodigo(c));
			}
			sb.append(" ");
		}
		System.out.println("\nMensaje en codigo Morse:\n"+ msjMorse);
		System.out.println("\nMensaje descifrado:\n"+ sb);
		System.out.println();
	}
	
	/**
	 * Metodo interno estatico para decodificar un fragmento de codigo Morse.
	 * @param codigo -- cadena de codigo Morse
	 * @return String -- fragmento de codigo morse decodificado
	 */
	private static String decodCodigo(String codigo) throws RuntimeException {
		NodoArbol pos = raiz;
		
		for(int i = 0; i < codigo.length(); i++) {
			if(codigo.charAt(i) == '.') {
				if(pos.izquierda!=null)
					pos = pos.izquierda;
				else
					throw new RuntimeException("Codigo invalido.");
			}
			
			if(codigo.charAt(i) == '-') {
				if(pos.derecha!=null)
					pos = pos.derecha;
				else
					throw new RuntimeException("Codigo invalido.");
			}
		}
		return (String)pos.valor;	
	}
	
	/**
	 * Metodo estatico para imprimir el contenido del arbol Morse en pre-orden.
	 */
	public static void imprimir() {
		preorden(raiz);
		System.out.println();
	}
	
	/**
	 * Metodo interno estatico para imprimir el arbol Morse en pre-orden.
	 * @param nodo -- nodo a partir del cual se realiza el recorrido
	 */
	private static void preorden(NodoArbol nodo) {
		if(nodo != null) {
			if(!nodo.valor.equals(""))
				System.out.print(nodo.valor + " ");
			preorden(nodo.izquierda);
			preorden(nodo.derecha);
		}
	}
	
	public static void main(String[] args) {
		
		if(args.length == 1) {
			String msjMorse = args[0];
			ArbolMorse.decodificar(msjMorse);
		}
		else
			System.out.println("Forma de ejecucion: java ArbolMorse \"mensajeEnMorse\"");
	}
}