import java.util.Scanner;
import java.io.File;
import java.io.FileNotFoundException;
/**
 * Clase para determinar la frecuencia de palabras en un archivo haciendo uso de un arbol binario de busqueda.
 * @author Yazmin Gonzalez Bayardo
 * @version Agosto 2015
 */
public class ContadorFrecuenciaPalabras {
	
	private ArbolBinarioBusqueda arbol;
	private String archivo;
	
	/**
	 * Constructor que recibe una cadena e inicializa un arbol contador de palabras.
	 * @param nombreArchivo - cadena con nombre del archivo sobre el que sea desea conocer la frecuencia de las palabras que lo componen.
	 */
	public ContadorFrecuenciaPalabras(String nombreArchivo) {
		arbol = new ArbolBinarioBusqueda(new ComparaPalabras());
		archivo = nombreArchivo;
	}
	
	/**
	 * Metodo para determinar la frecuencia de las palabras contenidas en el archivo. Imprime en pantalla el conteo de palabras obtenido.
	 */
	public void generarFrecuencia() {
		try{
			Scanner sc = new Scanner(new File(archivo));
			sc.useDelimiter("[^a-zA-Z0-9ñáéíóú]+");
			
			while(sc.hasNext()) {
				
				String p = sc.next().toLowerCase();
				Palabra palabra = new Palabra(p);
				
				Object elem = arbol.obtenerElemento(palabra);
					
				if(elem != null) 	// la palabra ya se encuentra en el arbol, solo se incrementa su frecuencia
					((Palabra)elem).incrementarFrec();
				else {	// es una nueva palabra
					palabra.incrementarFrec();
					arbol.agregar(palabra);
				}	
			}
			arbol.imprimirArbol(); // Imprime contenido del arbol en in-orden (orden alfabetico).
			
		}catch(FileNotFoundException e) {
			System.out.println("Archivo no encontrado.");
		}
	}
}

