/**
 * Clase de Ordenes para almacenar la información de las ordenes para la compra o venta de acciones.
 * @author  Yazmin Gonzalez Bayardo
 * @version Noviembre 2015
 */
public class Orden {
	private String usuario; // Nombre del usuario
	private String tipoOrden; // Tipo de orden: "comprar" o "vender"
	private int precio; // Precio de la accion
	private int numAcciones; // Numero de acciones por comprar o vender
	
	/**
	 * Constructor de una orden para comprar o vender acciones.
	 * @param u - nombre del usuario asociado a la orden de compra o venta.
	 * @param tipoO - cadena que indica el tipo de orden deseada que puede ser "comprar" o "vender".
	 * @param p - precio de la accion.
	 * @param numA - numero de acciones de la orden de compra o venta.
	 */
	public Orden(String u, String tipoO, int p, int numA) {
		usuario = u;
		tipoOrden = tipoO;
		precio = p;
		numAcciones = numA;
	}
	
	/**
	 * Metodo para obtener el nombre del usuario.
	 * @return String - nombre del usuario asociado a la orden de compra o venta.
	 */
	public String obtenerUsuario() {
		return usuario;
	}
	
	/**
	 * Metodo para obtener el tipo de orden.
	 * @return String - devuelve la cadena "comprar" o "vender" segun sea el caso.
	 */
	public String obtenerTipoOrden() {
		return tipoOrden;
	}
	
	/**
	 * Metodo para obtener el precio por accion de la orden.
	 * @return int - precio de la accion.
	 */
	public int obtenerPrecio() {
		return precio;
	}
	
	/**
	 * Metodo para obtener el numero de acciones de una orden de compra o venta.
	 * @return int - numero de acciones.
	 */
	public int obtenerNumAcciones() {
		return numAcciones;
	}
	
	/**
	 * Metodo para asignar el nombre del usuario.
	 * @param u - nombre del usuario asociado a la orden de compra o venta.
	 */
	public void asignarUsuario(String u) {
		usuario = u;
	}
	
	/**
	 * Metodo para asignar el tipo de orden.
	 * @param tipoO - cadena que indica el tipo de orden deseada que puede ser "comprar" o "vender".
	 */
	public void asignarTipoOrden(String tipoO) {
		tipoOrden = tipoO;
	}
	
	/**
	 * Metodo para asignar el precio de una accion.
	 * @param p - precio de la accion.
	 */
	public void asignarPrecio(int p) {
		precio = p;
	}
	
	/**
	 * Metodo para asignar el numero de acciones de la orden de compra o venta.
	 * @param numA - numero de acciones de la orden de compra o venta.
	 */
	public void asignarNumAcciones(int numA) {
		numAcciones = numA;
	}
	
	/**
	 * Metodo para convertir una Orden a una cadena de caracteres.
	 * @return String - la orden en formato de cadena.
	 */
	public String toString() {
		return "Usuario: " + usuario + "  Tipo de orden: " + tipoOrden + "  Acciones: " + numAcciones +"  Precio: $"+ precio;
	}
}