import java.io.StreamTokenizer;
import java.io.FileReader;
import java.util.Scanner;
import java.util.Iterator;
/**
 * Programa para buscar palabras en un archivo de texto.
 * @author Yazmin Gonzalez Bayardo
 * @version Diciembre 2015
 */
public class BuscadorDePalabras {
	
	private TablaDeDispersion tabla;
	private static Scanner in;
	
	/**
	 * Constructor de un buscador de palabras en un archivo.
	 * @param archivo -- nombre del archivo.
	 */
	public BuscadorDePalabras(String archivo) {
		tabla = new TablaDeDispersion();
		llenarTabla(archivo);
	}
	
	/**
	 * Metodo privado que procesa el archivo y llena la tabla de dispersion.
	 * @param archivo -- nombre del archivo por procesar.
	 */
	private void llenarTabla(String archivo) {
		try{
			
			StreamTokenizer stk = new StreamTokenizer(new FileReader(archivo));
			
			stk.lowerCaseMode(true);
			stk.ordinaryChar('.');
			int p = stk.nextToken();
			Palabra palabra;
			Lista listaLineas;
			
			while (p != StreamTokenizer.TT_EOF) {
				if (p == StreamTokenizer.TT_WORD || p == StreamTokenizer.TT_NUMBER) {
					
					if(stk.ttype == StreamTokenizer.TT_NUMBER){
						if((int)stk.nval == stk.nval)
							palabra = new Palabra(Integer.toString((int)stk.nval));
						else
							palabra = new Palabra(Double.toString(stk.nval));
					}
					else
						palabra = new Palabra(stk.sval);
					
					if(tabla.contiene(palabra))
						palabra = (Palabra)tabla.obtener(palabra).buscar(palabra).elemento;
					else 
						tabla.agregar(palabra);
					
					palabra.agregarLinea(stk.lineno());   
				}
			    p = stk.nextToken();
			}
		} catch (Exception e) {
			System.out.println("Error "+e);
		}
	}
	
	/**
	 * Menu de opciones para buscar palabras.
	 */
	private void menu() {
		System.out.println("\nElige una opcion");
		System.out.println("1. Buscar");
		System.out.println("0. Terminar");
	}
	
	/**
	 * Metodo privado para realizar la accion elegida del menu.
	 * @param opcion -- opcion de la accion elegida por el usuario.
	 */
	private void accion(int opcion) {
		switch(opcion) {
			case 0:
				break;
			case 1: 
				System.out.println("Introduce la palabra:");
				buscar(in.next());
				break;
			default:
				System.out.println("Opcion invalida\n");
				break;
		}
	}
	
	/**
	 * Metodo para buscar una palabra.
	 * @param palabra-- cadena de la palabra que se busca en el archivo.
	 */
	public void buscar(String palabra) {
		Palabra p = new Palabra(palabra.toLowerCase());
		if(tabla.contiene(p)) {
			p = (Palabra)tabla.obtener(p).buscar(p).elemento;
			Lista lineas = p.obtenerLineas();
			Iterator it = lineas.iterador();
			System.out.println("\nSe encontro la palabra \""+p+ "\" en la(s) linea(s): ");
			while(it.hasNext())
				System.out.print(it.next()+" ");
			System.out.println();
		}
		else {
			System.out.println("\nNo se encontro la palabra \""+p+ "\".");
		}		
	}
	
	public static void main(String[] args) {
		int opcion;
		in = new Scanner(System.in);
		
		if(args.length!=1) {
			System.out.println("Forma de uso: java BuscadorDePalabras nombreArchivo");
			System.exit(0);
		}
		
		try{
			BuscadorDePalabras buscador = new BuscadorDePalabras(args[0]);
			do {
				buscador.menu();
				opcion = in.nextInt();
				buscador.accion(opcion);
			} while(opcion!=0);
		} catch(Exception e){
			System.out.println("Error: "+e);
		}
		System.out.println("¡Hasta luego!");
	}
}