/**
 * Clase para trabajar con palabras.
 * @author Yazmin Gonzalez Bayardo
 * @version Diciembre 2015
 */
public class Palabra {
	
	private String palabra;
	private Lista lineas;
	
	/**
	 * Constructor de una palabra.
	 * @param palabra -- cadena de la palabra.
	 */
	public Palabra(String palabra) {
		this.palabra = palabra;
		lineas = new Lista();
	}
	
	/**
	 * Metodo para obtener la palabra.
	 * @return String -- la palabra.
	 */
	public String obtenerPalabra() {
		return palabra;
	}
	
	/**
	 * Metodo para obtener la lista de las lineas donde se encontro la palabra.
	 * @return Lista -- lista de lineas.
	 */
	public Lista obtenerLineas() {
		return lineas;
	}
	
	/**
	 * Metodo para convertir una Palabra en una cadena de caracteres.
	 * @return String -- la palabra en formato de cadena.
	 */
	public String toString() {
		return palabra;
	}
	
	/**
	 * Metodo para agregar un numero de linea a la lista de lineas de la Palabra.
	 * @param linea -- numero de linea.
	 */
	public void agregarLinea(int linea) {
		lineas.agregar(linea);
	}
	
	/**
	 * Metodo para determinar si dos palabras son iguales.
	 * @param o -- palabra contra la cual se va a comparar.
	 * @return boolean -- true si son iguales y false en otro caso.
	 */
	public boolean equals(Object o) {
		if(o == this) return true;
		if(o == null) return false;
		if(o instanceof Palabra) {
			Palabra p = (Palabra)o;
			return this.palabra.equals(p.obtenerPalabra());
		}
		return false;
	}
	
	/**
	 * Metodo para obtener un valor de dispersión para una palabra.
	 * @return int -- valor de dispersion de la palabra.
	 */
	public int hashCode() {
		int valor = 0;
		for(int i = 0; i < palabra.length(); i++)
			valor = 37*valor + palabra.charAt(i);
		return valor;
	}

}