/**
 * Programa que implementa el tipo abstracto de datos Polinomio.
 * Pn(x) = AnX^n + An-1X^n-1 + ... + A2X^2 + A1X + A0
 * @author Yazmin Gonzalez Bayardo
 * @version Febrero 2015
 */
public class Polinomio implements PolinomioTAD {
	
	private int[] coef;
	private int grado;
	
	/**
     * Constructor por omision de un polinomio
     */
	public Polinomio() {
		this(0,0);
	}
	
	/**
     * Constructor de un polinomio a * x^b
	 * @param a -- coeficiente del polinomio
	 * @param b -- exponente del polinomio
     */
	public Polinomio(int a, int b) {
		coef = new int[b+1];
		coef[b] = a;
		grado = grado();
	}
	
	/**
     * Constructor de copia.
	 * @param p -- polinomio que se usa para crear uno nuevo 
     */
	public Polinomio(Polinomio p) {
		coef = new int[p.coef.length];
		for(int i = 0; i < coef.length; i++)
			coef[i] = p.coef[i];
		grado = p.grado;
	}
	
    /**
     * Determina el grado de un polinomio
     * @return int - grado del polinomio
     */
	public int grado() {
		int g = 0;
		for(int i = 0; i < coef.length ; i++)
			if(coef[i] != 0)
				g = i;
		return g;
	}
	
    /**
     * Devuelve la suma de dos polinomios
	 * @param p - Polinomio que sera sumado al que llama este metodo
     * @return Polinomio - Polinomio que resulta de la suma
     */
	public Polinomio sumar(PolinomioTAD p) {
		Polinomio p1 = new Polinomio(this);
		Polinomio p2 = (Polinomio) p;
		Polinomio poliSuma = new Polinomio(0, (p1.grado>p2.grado)? p1.grado : p2.grado);
			
		for(int i = 0; i <= p1.grado; i++)
			poliSuma.coef[i] += p1.coef[i];
		for(int i = 0; i <= p2.grado ; i++)
			poliSuma.coef[i] += p2.coef[i];
		
		
		poliSuma.grado = poliSuma.grado();
		
		return poliSuma;		
	}
	
    /**
     * Devuelve la resta de dos polinomios
	 * @param p - Polinomio que sera restado al que llama este metodo
     * @return Polinomio - Polinomio que resulta de la resta
     */
	public Polinomio restar(PolinomioTAD p) {
		Polinomio p1 = new Polinomio(this);
		Polinomio p2 = (Polinomio) p;
		Polinomio poliResta = new Polinomio(0, (p1.grado>p2.grado)? p1.grado : p2.grado);
		
		for(int i = 0; i <= p1.grado; i++)
			poliResta.coef[i] += p1.coef[i];
		for(int i = 0; i <= p2.grado ; i++)
			poliResta.coef[i] -= p2.coef[i];
		
		poliResta.grado = poliResta.grado();
		
		return poliResta;		
	}
	
    /**
     * Devuelve la multiplicacion de dos polinomios
	 * @param p - Polinomio que sera multiplicado al que llama este metodo
     * @return Polinomio - Polinomio que resulta de la multiplicacion
     */
	public Polinomio multiplicar(PolinomioTAD p) {
		Polinomio p1 = new Polinomio(this);
		Polinomio p2 = (Polinomio) p;
		Polinomio poliMult = new Polinomio(0, p1.grado + p2.grado);
		
		for(int i = 0; i <= p1.grado; i++)
			for(int j = 0; j <= p2.grado ; j++)
				poliMult.coef[i+j] += (p1.coef[i] * p2.coef[j]);
		
		poliMult.grado = poliMult.grado();
		
		return poliMult;		
	}
	
    /**
     * Imprime en pantalla los terminos que conforman al polinomio
     */
	public void imprimir() {		
		if (grado == 0)
			System.out.println(coef[0]);
		else {
			String polinomio="";
			if(grado == 1)
				polinomio += coef[grado] + "x";
			else
				polinomio += coef[grado] + "x^" + grado;
			
			for (int i = grado-1; i >= 0; i--) {
				if(coef[i]!=0){
					if (coef[i] > 0)
						polinomio +=  " + " + (coef[i]);
					if (coef[i] < 0)
						polinomio +=  " - " + (-coef[i]);
				
					if(i == 1)
						polinomio+= "x";
					else if (i >  1)
						polinomio += "x^" + i;
				}
			}
			System.out.println(polinomio);
		}				
	}
	
    /**
     * Devuelve el polinomio evaluado
	 * @param x - valor por evaluar en el polinomio
     * @return double - resultado de la evaluacion
     */
	public double evaluar(double x) {
		double e = 0;
		for(int i = grado; i >= 0; i--)
			e += coef[i] * (Math.pow(x,i));
		return e;
	}
	
}