import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Programa para iterar sobre un arreglo de cadenas 
 * @author Yazmin Gonzalez Bayardo
 * @version Febrero 2015
 */
public class ArregloPalabras {
	
	private String[] palabras; // Arreglo para almacenar palabras
	
	/**
	 * Constructor de un arreglo de palabras
	 * @param p - Arreglo de cadenas
	 */
	public ArregloPalabras(String[] p) {
		palabras = p;
	}
	
	/** 
	 * Devuelve un iterador sobre las palabras del arreglo
	 * @return Iterator
	 */
	public Iterator iterador() {
		return new IteradorPalabras();
	}
	
	// Clase privada interna que implementa la interfaz Iterator
	private class IteradorPalabras implements Iterator {
		
		private int index; //Indice para recorrer el arreglo de palabras
		
		// Constructor por omision
		public IteradorPalabras () {
			index = 0;
		}
		
		// Determina si todavia hay una palabra que comience con la letra 'A' en el arreglo
		public boolean hasNext() {
			while(index<palabras.length && (palabras[index]==null || (palabras[index].length()<=0) || (palabras[index].charAt(0)!='A')))
				index++;
			return index<palabras.length;
		}
		
		// Devuelve la siguiente palabra en el arreglo
		public String next() {
			if(hasNext())
				return palabras[index++];
			throw new NoSuchElementException("No hay elementos en la estructura.");
		}
		
		// Metodo no soportado
		public void remove() {
			throw new UnsupportedOperationException();
		}
	}
}
