import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Programa que extiende la clase Bolsa e incluye la funcion para iterar sobre los elementos de la coleccion.
 * @author Yazmin Gonzalez Bayardo
 * @version Febrero 2015
 */
public class BolsaConIterador extends Bolsa {
	
	// Constructor por omision.
	public BolsaConIterador() {
		super();
	}
	
	/**
     * Crea un iterador para recorrer los elementos de la bolsa
     * @return Iterator - iterador sobre la coleccion
     */
	public Iterator iterador() {
		return new IteradorBolsa(super.numElem, super.bolsa);
	}
	
	// Clase privada interna que implementa la interfaz Iterator
	private class IteradorBolsa implements Iterator {
		
		private int index; // Indice para recorrer los elementos de la bolsa
		private int numElem;
		private Object[] bolsa;
		
		public IteradorBolsa(int num, Object[] b) { // Constructor por omision
			index = 0;
			numElem = num;
			bolsa = b;
		}
		
		// Determina si hay mas elementos en la bolsa
		public boolean hasNext() {
			return index < numElem;
		}
		
		// Devuelve el siguiente elemento en la bolsa
		public Object next() {
			if(hasNext())
				return bolsa[index++];
			throw new NoSuchElementException("No hay elementos en la bolsa.");
		}
		
		// Metodo no soportado
		public void remove() {
			throw new UnsupportedOperationException();
		}
	}
}