/**
 * Clase Tarea utilizada para probar la clase de listas ordenadas
 * @author Yazmin Gonzalez Bayardo
 * @version Marzo 2015
 */
public class Tarea {
	
    private String tarea; // Cadena para especificar la tarea por realizar
	private int prioridad; // Prioridad de la tarea para llevarla a cabo
    
	/**
     * Constructor de una Tarea a partir de su texto y prioridad
     * @param tarea -- texto que especifica la tarea
	 * @param prioridad -- prioridad de la tarea
	 * @throws IllegalArgumentException -- si la prioridad es menor o igual a 0
     */
    public Tarea(String tarea, int prioridad) {
		if(prioridad <= 0) {
			throw new IllegalArgumentException("La prioridad es menor o igual a cero.");
		}
		this.tarea = tarea;
		this.prioridad = prioridad;
    }
	
	/**
     * Constructor de una Tarea a partir de su texto. La prioridad por omision es 1.
     * @param tarea -- texto que especifica la tarea
     */
    public Tarea(String tarea) {
		this(tarea,1);
    }

    /**
     * Metodo para obtener la tarea
     * @return String -- tarea
     */
    public String obtenerTarea() {
		return tarea;
    }
	
    /**
     * Metodo para obtener la prioridad de una tarea
     * @return int -- prioridad de la tarea
     */
    public int obtenerPrioridad() {
		return prioridad;
    }
	
    /**
     * Metodo para asignar el texto de la tarea
     * @param tar -- texto de la tarea
     */
    public void asignarTarea(String tar) {
		tarea = tar;
    }
	
    /**
     * Metodo para asignar la prioridad de una tarea
     * @param prior -- prioridad de la tarea
     */
    public void asignarPrioridad(int prior) {
		prioridad = prior;
    }
	
    /**
     * Metodo para mostrar una tarea en forma de cadena
     * @return String  -- datos de la tarea
     */
    public String toString() {
		return "- " + tarea + ", prioridad: " + prioridad;
    }
	
    /**
     * Metodo para determinar si dos tareas son iguales. Si las cadenas de las tareas son iguales sin importar minusculas/mayusculas, entonces
	 * las tareas son iguales.
	 * @param o -- Tarea contra el cual se va a comparar
     * @return boolean  -- true si las tareas son iguales, false en otro caso
     */
    public boolean equals(Object o) {
		if(o == this) return true;
		if(o == null) return false;
		if(o instanceof Tarea) {
			Tarea tar = (Tarea) o;
			return (obtenerTarea().toLowerCase().trim().equals(tar.obtenerTarea().toLowerCase().trim()));
		}
		return false;
    }
}