import java.util.Comparator;
import java.util.Iterator;
/**
 * Clase para crear una lista cuyos elementos esten ordenados
 * @author Yazmin Gonzalez Bayardo
 * @version Marzo 2015
 */
public class ListaOrdenada extends Lista {
	
	private Comparator<Object> cmp;
	
    /**  
     * Metodo para construir una lista ordenada
	 * @param cmp -- comparador para los elementos de la lista
     */
	public ListaOrdenada(Comparator<Object> cmp){
		super();
		this.cmp = cmp;
	}
 
    /**
     * Metodo para insertar un elemento a la lista ordenada.
     * @param dato -- el dato a agregar de manera ordenada.
     */	
	@Override
	public void agregar(Object dato) {
		
		if(dato == null)
			return;
		
		if (estaVacia())
			inicio = fin = new Nodo(dato);
		else {
			if(cmp.compare(inicio.elemento,dato) > 0) // El dato debe ser el inicio de la lista
				inicio = new Nodo(dato,inicio);
			else {
				if(cmp.compare(fin.elemento,dato) < 0) // El dato debe ser el fin de la lista
					fin = fin.sgte = new Nodo(dato);
				else {
					// El dato debe ir en medio de la lista
					Nodo pos = inicio;
					Nodo anterior = null;
					
					while((pos!=null) && (cmp.compare(pos.elemento,dato) <= 0))
					{
						anterior = pos;
						pos = pos.sgte;
					}
					
					if(pos == null)
						fin = anterior.sgte = new Nodo(dato);
					else
						anterior.sgte = new Nodo(dato,pos);
				}
			}
		}
	}
	
    /**
     * Metodo para intercalar los elementos de dos listas.
     * @param lis -- lista cuyos elementos se intercalaran con los de la lista ordenada que llama al metodo.
     */
	public void fundir(Lista lis) {
		
		if(lis.estaVacia())  //Regresa la lista ordenada que llama al metodo en caso que la lista pasada como parametro esta vacia
			return;
		else{
			
			if(lis instanceof ListaOrdenada) { // Si la lista pasada como parametro esta ordenada, intercala los elementos de las listas ordenadas
				Lista listaTmp = new Lista(); // Lista temporal donde se fusionan las listas
				Nodo pos = inicio;
				Nodo posLis = lis.inicio;
			
				while( pos != null && posLis != null) {
					
					if(cmp.compare(pos.elemento,posLis.elemento) < 0) {
						listaTmp.agregar(pos.elemento);
						pos = pos.sgte;
					}else if(cmp.compare(pos.elemento,posLis.elemento) > 0) {
						listaTmp.agregar(posLis.elemento);
						posLis = posLis.sgte;
					}else {
						listaTmp.agregar(pos.elemento);
						pos = pos.sgte;
						listaTmp.agregar(posLis.elemento);
						posLis = posLis.sgte;
					}
				}
			
				while (pos!=null) {
					listaTmp.agregar(pos.elemento);
					pos = pos.sgte;
				}
			
				while (posLis!=null) {
					listaTmp.agregar(posLis.elemento);
					posLis = posLis.sgte;
				}
			
				lis.vaciar();
				vaciar();
				inicio = listaTmp.inicio;
				fin = listaTmp.fin;
				listaTmp.vaciar();
			}
			else { 								// Si la lista pasada como parametro no esta ordenada
				for(Iterator it = lis.iterador(); it.hasNext();) {
					agregar(it.next());			// Cada elemento de la lista se inserta de manera ordenada a la lista que llama al metodo
				}
				lis.vaciar();
			}
		}
	}
	
    /**
     * Notifica que el metodo para agregar un dato al inicio de una lista ordenada no aplica.
     * @param dato -- el dato a agregar.
     */
	@Override
	public void agregarAlInicio(Object dato) {
		System.out.println("El metodo agregarAlInicio no aplica para la Lista Ordenada.");
		return;
    }
	
    /**
	 * Notifica que el metodo para sustituir el valor actual de un nodo por otro nuevo, no aplica para una lista ordenada.
     * @param orig -- valor original
     * @param nuevo -- nuevo valor
     */
	@Override
    public void sustituir(Object orig, Object nuevo)  {
		System.out.println("El metodo sustituir no aplica para la Lista Ordenada.");
		return;
	}
}