import java.io.Serializable;
/**
 * Clase para trabajar con peliculas. Los objetos de la clase pueden serializarse.
 * @author Yazmin Gonzalez Bayardo
 * @version Marzo 2015
 */
public class Pelicula implements Serializable{
	
	private String titulo;
	private int calificacion;
	
	/**
	 * Constructor de una pelicula a partir del titulo y su calificacion
	 * @param titulo - titulo de la pelicula
	 * @param calificacion - calificacion de la pelicula
	 * @throws IllegalArgumentException - si la calificacion no es un entero dentro del rango 1 a 10
	 */
	public Pelicula(String titulo, int calificacion){
		if(calificacion <= 0 || calificacion > 10 ) {
			throw new IllegalArgumentException("Pelicula con calificacion fuera del rango 1 a 10");
		}
		this.titulo = titulo;
		this.calificacion = calificacion;
	}
	
	
	/**
	 * Constructor de una pelicula a partir del titulo. La calificacion por omisión es 1.
	 * @param titulo - titulo de la pelicula
	 */
	public Pelicula(String titulo){
		this (titulo, 1);
	}
	
	/**
	 * Metodo para asignar el titulo de la pelicula
	 * @param nombre - titulo de la pelicula
	 */
	public void asignarTitulo(String nombre) {
		titulo = nombre;
	}
	
	/**
	 * Metodo para asignar la calificacion de la pelicula
	 * @param calif - calificacion de la pelicula
	 */
	public void asignarCalificacion(int calif) {
		calificacion = calif;
	}
	
	/**
	 * Metodo para obtener el titulo de la pelicula
	 * @return String - titulo de la pelicula
	 */
	public String obtenerTitulo() {
		return titulo;
	}
	
	/**
	 * Metodo para obtener la calificacion de la pelicula
	 * @return int - calificacion de la pelicula
	 */
	public int obtenerCalificacion() {
		return calificacion;
	}
	
	/**
	 * Metodo para determinar si dos peliculas son iguales.
	 * @param p - pelicula contra la cual se va a comparar
	 * @return boolean - true si son iguales y false en otro caso
	 */
	public boolean equals(Object p) {
		Pelicula peli = (Pelicula)p;
		return (titulo.toLowerCase().trim()).equals(peli.obtenerTitulo().toLowerCase().trim());
	}
	
	/**
	 * Metodo para convertir una Pelicula en una cadena de caracteres
	 * @return String - la pelicula en formato de cadena
	 */
	public String toString() {
		return titulo + "          " + calificacion;
	}
}