import java.util.NoSuchElementException;
/**
 * Implementacion de la interfaz PilaDoble para crear dos pilas en un arreglo
 * @author Yazmin Gonzalez Bayardo
 * @version Abril 2015
 */
public class PilasConArreglo implements PilaDoble {
	
	private Object[] arreglo;	// Arreglo para contener a 2 pilas
	private int topePila1;		// Tope de la pila 1
	private int topePila2;		// Tope de la pila 2
	
    /**
     * Constructor por omision
     */
	public PilasConArreglo() {
		this(10);
	}
	
    /**
     * Constructor de dos pilas en un arreglo
	 * @param capacidad -- tamanio del arreglo
	 * @throws IllegalArgumentException -- la capacidad del arrego es menor a 2
     */
	public PilasConArreglo(int capacidad) {
		
		if(capacidad < 2)
			throw new IllegalArgumentException("La capacidad del arreglo debe ser mayor o igual a 2.");
		
		arreglo = new Object[capacidad];
		topePila1 = -1;
		topePila2 = capacidad;
	}
	
    /**
     * Determina si la pila indicada como parametro esta vacia.
	 * @param pilaId -- identificador de la pila
     * @return boolean -- true si la pila esta vacia y false en otro caso
	 * @throws IllegalArgumentException -- identificador de pila invalido
     */
	public boolean estaVacia (int pilaId) {
		
		if(pilaId == 1)
			return (topePila1 == -1);
		else {
			if(pilaId == 2)
				return (topePila2 == arreglo.length);
			else
				throw new IllegalArgumentException("Identificador de pila invalido.");
		}
	}
	
    /**
     * Vacia la pila indicada como parametro.
	 * @param pilaId -- identificador de la pila
	 * @throws IllegalArgumentException -- identificador de pila invalido
     */
	public void vaciar(int pilaId) {
		if(pilaId == 1)
			topePila1 = -1;
		else {
			if(pilaId == 2)
				topePila2 = arreglo.length;
			else
				throw new IllegalArgumentException("Identificador de pila invalido.");
		}
	}
	
	/**
	 * Devuelve el elemento del tope de la pila indicada como parametro sin alterar la pila.
	 * @param pilaId -- identificador de la pila
	 * @return Object -- objeto del tope de la pila
	 * @throws IllegalArgumentException -- identificador de pila invalido
	 */
	public Object top(int pilaId) {
		
		if(pilaId == 1) {
			return (estaVacia(1))? null: arreglo[topePila1];
		}	
		else {
			if(pilaId == 2) {
				return (estaVacia(2))? null: arreglo[topePila2];
			}else
				throw new IllegalArgumentException("Identificador de pila invalido.");
		}
	}
	
	/**
	 * Extrae el elemento del tope de la pila indicada como parametro.
	 * @param pilaId -- identificador de la pila
	 * @return Object -- objeto del tope de la pila
	 * @throws IllegalArgumentException -- identificador de pila invalido
	 */
	public Object pop(int pilaId) {
		
		if(pilaId == 1) {
			
			if (estaVacia(1))
			    return null;
			
			Object resul = arreglo[topePila1];
			arreglo[topePila1] = null;
			topePila1--;
			return resul;
		}
		else {
			if(pilaId == 2) {
				
				if (estaVacia(2))
				    return null;
				
				Object resul = arreglo[topePila2];
				arreglo[topePila2] = null;
				topePila2++;
				return resul;
			}else
				throw new IllegalArgumentException("Identificador de pila invalido.");
		}
	}
	
	/**
	 * Inserta el elemento en el tope de la pila indicada como parametro.
	 * @param elem -- elemento a insertar en la pila
	 * @param pilaId -- identificador de la pila
	 * @throws IllegalArgumentException -- identificador de pila invalido
	 */
	public void push(int pilaId, Object elem) {
		
		if(pilaId == 1) {
			if((topePila1+1) == topePila2)
				System.out.println("El arreglo se encuentra lleno (pila 1).");
			else
				arreglo[++topePila1] = elem;
		}
		else {
			if(pilaId == 2) {
				if((topePila2-1) == topePila1)
					System.out.println("El arreglo se encuentra lleno (pila 2).");
				else
					arreglo[--topePila2] = elem;
			}
			else
				throw new IllegalArgumentException("Identificador de pila invalido.");
		}
	}
	
	/**
     * Crea un iterador para recorrer los elementos de la pila indicada como parametro
     * @return Iterator - iterador sobre la pila
     */
	public java.util.Iterator iterador(int pilaId) {
		return new MiIterador(pilaId);
	}
	
	// Clase privada interna que implementa la interfaz Iterator
	private class MiIterador implements java.util.Iterator {
		
		private int posicion; // Indice para recorrer los elementos de la bolsa
		private int pilaId; // Identificador de la pila
		
		// Constructor
		public MiIterador(int pilaId) {
			this.pilaId = pilaId;
			if(pilaId == 1) {
				posicion = topePila1;				
			}
			else {
				if(pilaId == 2) {
   				 	posicion = topePila2;
   			 	}
   			 	else
   					throw new IllegalArgumentException("Identificador de pila invalido");
			}
		}
		
		// Determina si hay mas elementos en la pila 
		public boolean hasNext() {
			if(pilaId == 1)
				return posicion > -1 && arreglo[posicion] != null;
			else
				return posicion < arreglo.length && arreglo[posicion] != null;
		}
		
		// Devuelve el siguiente elemento en la pila
		public Object next() {
			if(hasNext()) {
				Object o = arreglo[posicion];
				if(pilaId == 1)
					posicion--;
				else
					posicion++;
				
				return o;
			}
			throw new NoSuchElementException();
		}
		
		// Metodo no soportado
	 	public void remove() {
	 	    throw new UnsupportedOperationException();
	 	}
	 }
}







