import java.util.Scanner;
/**
 * Programa para simular el comportamiento de navegacion en paginas web
 * @author Yazmin Gonzalez Bayardo
 * @version Mayo 2015
 */
public class Navegador {

	private Pila atras; // Pila para almacenar paginas anteriores
	private Pila adelante; // Pila para almacenar paginas posteriores
	
	private int idPagina = 0; 
	private static Scanner in = new Scanner(System.in);
	
    /**
     * Constructor para simular comportamiento de navegacion
     */
	public Navegador() {
		atras = new Pila();
		adelante = new Pila();
	}
	
    /**
     * Metodo para navegar a la pagina anterior visitada.
     * @return true si hay pagina anterior y falso en otro caso.
     */
	public boolean regresar() {
		if(!atras.estaVacia()) {
			if(atras.tamanio() > 1) {
				adelante.push(atras.pop());
				return true;
			}else
				return false;	
		}else
			return false;
	}
	
    /**
     * Metodo para navegar a la pagina siguiente visitada.
     * @return true si hay pagina siguiente y falso en otro caso.
     */
	public boolean adelantar() {
		if(!adelante.estaVacia()) {
			atras.push(adelante.pop());
			return true;
		}else
			return false;
	}
	
    /**
     * Metodo para introducir la url de una nueva pagina que se visita.
     * @param url - cadena con la url a visitar
     */
	public void nuevaUrl(String url) {
		atras.push(url);
		adelante.vaciar();
	}
	
    /**
     * Metodo que simula la accion de un clic sobre liga de una pagina que se visita. 
	 * Genera una cadena correspondiente a una url.
     */
	public void clicLiga() {
		idPagina++;
		atras.push("http://www.liga" + idPagina + ".com");
		adelante.vaciar();
	}
	
    /**
     * Metodo para mostrar la pagina actual que se despliega en el navegador.
     */
	public void mostrarPagina() {
		if(atras.top() != null)
			System.out.println("\nPagina actual: " + atras.top());
	}
	
	
	/**
	 * Menu de opciones para trabajar con el navegador.
	 */
	public void menu() {
		System.out.println("\nElige una opcion");
		System.out.println("1. Boton Regresar");
		System.out.println("2. Boton Avanzar");
		System.out.println("3. Ingresar URL");
		System.out.println("4. Clic a liga");
		System.out.println("0. Terminar");
	}
	
	
	/**
	 * Metodo para ejecutar las acciones indicadas al navegador.
	 * @param opcion - opcion elegida por el usuario
	 */
	public void navegar(int opcion) {
		String url;
		switch(opcion) {
			case 0: break;
			case 1: if(regresar()) {
						mostrarPagina();
					}
					break;
			case 2: if(adelantar()) {
						mostrarPagina();
					}
					break;
			case 3: System.out.println("Introduce la url");
					url = in.nextLine();
					nuevaUrl(url);
					mostrarPagina();
					break;
			case 4:
					clicLiga();
					mostrarPagina();
					break;
			default:
					System.out.println("Opcion invalida\n");
		}
	}
	
	public static void main(String[] args) {
		Navegador miNavegador = new Navegador();
		int opcion;
			
		try{
			do{
				miNavegador.menu();
				opcion = Integer.parseInt(in.nextLine());
				miNavegador.navegar(opcion);
			}while(opcion!=0);
		} catch(Exception e) {
			System.out.println(e);
		}
	}
}