/**
 * Clase para almacenar la informacion de un contenedor
 * @author Yazmin Gonzalez Bayardo
 * @version Mayo 2015
 */
public class Contenedor {

	private final int numId;
	private static int registro = 100;
	private double peso;
	
	/**
	 * Constructor que recibe los datos del contenedor
	 * @param peso - peso en kg del contenedor
	 */
	public Contenedor(double peso) {
		this.numId = ++registro;
		this.peso = peso;
	}
	
	/**
	 * Metodo para obtener el identificador del contenedor
	 * @return int - numero identificador del contenedor
     */
	public int obtenerNumId() {
		return numId;
	}
	
	/**
	 * Metodo para obtener el peso del contenedor
	 * @return double - peso en kg del contenedor
     */
	public double obtenerPeso() {
		return peso;
	}
	
	/**
	 * Metodo para asignar el peso del contenedor
	 * @param peso - peso en kg del contenedor
     */
	public void asignarPeso(double peso) {
			this.peso = peso;
	}
	
	/**
	  * Metodo para mostrar la informacion de un contendor
	  * @return String - cadena con la informacion del contenedor
	  */
	public String toString() {
		return "Identificador del contenedor: " + numId + " Peso: " + peso + " Kg.";
	}
	
	/** 
	  * Determina si dos contenedores son iguales, es decir, si tienen el mismo identificador
	  * @param o - contenedor contra el cual se va a comparar
	  * @return boolean - true si son iguales y false en otro caso
	  */
	public boolean equals(Object o) {
		if (o == this) return true;
		if (o == null) return false;
		if (o instanceof Contenedor) {
			Contenedor c = (Contenedor)o;
			return obtenerNumId() == c.obtenerNumId();
		}
		return false;
	}
}