import java.util.Comparator;
/**
 * Implementacion de una pila de objetos que incluye una funcion de complejidad constante que devuelve el menor de los objetos
 * @author Yazmin Gonzalez Bayardo 
 * @version Mayo 2015
 */
public class PilaConMinimo extends Pila {
	
	private Pila auxiliar; // Pila auxiliar para almacenar los objetos minimos
	private Comparator<Object> cmp;
	
    /**
     * Constructor de la pila 
	 * @param cmp - comparador de objetos
     */
    public PilaConMinimo(Comparator<Object> cmp) {
		super();
		auxiliar = new Pila();
		this.cmp = cmp;
    }
	
    /**
     * Inserta un nuevo elemento en la pila mientras que almacena en la pila auxiliar el numero minimo que se conozca hasta el momento.
     * @param x - elemento a insertar.
     */
    public void push(Object x) {
		super.push(x);
		if(auxiliar.estaVacia()) {
			auxiliar.push(x);
		}
		else {
			Object topeAux = auxiliar.top();
			if(cmp.compare(x,topeAux) < 0) // El numero por insertar es menor que el minimo actual, por lo que se incluye como el nuevo numero minimo.
				auxiliar.push(x);
			else
				auxiliar.push(topeAux); // El numero minimo sigue siendo el minimo actual.
		}
    }
	
    /**
     * Extrae el elemento del tope de la pila mientras actualiza la pila auxiliar de objetos minimos.
     * Devuelve null si la pila esta vacia.
     * @return Object - objeto del tope de la pila
     */
    public Object pop() {
		if (this.estaVacia()){
			auxiliar.vaciar();
			return null;
		}
	    	
		auxiliar.pop(); // Para actualizar el objeto minimo siguiente.
		return super.pop();
    }
	
    /**
     * Vacia la pila y la pila auxiliar.
     */
    public void vaciar() { 
		super.vaciar();
		auxiliar.vaciar();
    }
	
    /**
     * Obtiene el elemento correspondiente al numero minimo de la pila (sin alterar esta).
     * @return Object - elemento con valor minimo de la pila. Devuelve null si la pila esta vacia.
     */
    public Object minimo() {
		if (auxiliar.estaVacia())
			return null;
		else
			return auxiliar.top();
    }
}