import java.util.NoSuchElementException;
/**
 * Clase que implementa el TAD Cola Circular usando nodos ligados.
 * @author Yazmin Gonzalez Bayardo
 * @version Junio 2015
 */
public class ColaCircular implements InterfazCola {
	
	protected Nodo ultimo; // Nodo del final de la cola circular.
	protected int nDatos; // Cantidad de elementos en la cola circular.
	
	/**
	 * Constructor por omision. Construye la cola circular.
	 */
	public ColaCircular() {
		ultimo = null;
		nDatos = 0;
	}
	
	/**
	 * Metodo para determinar si la cola circular esta vacia o no.
	 * @return boolean -- true si esta vacia y false en otro caso.
	 */
	public boolean estaVacia() {
		return ultimo == null;
	}
	
	/**
	 * Metodo para dejar vacia la cola circular.
	 */
	public void vaciar() {
		ultimo = null;
		nDatos = 0;
	}
	
	/**
	 * Metodo para conocer el tamanio de la cola circular.
	 * @return int -- cantidad de elementos en la cola circular.
	 */
	public int tamanio() {
		return nDatos;
	}
	
	/**
	 * Metodo para insertar un elemento a la cola circular.
	 * @param valor -- elemento que sera insertado.
	 */
	public void agregar(Object valor) {
		if(estaVacia()){
			ultimo = new Nodo(valor);
			ultimo.sgte = ultimo;
		}
		else {
			Nodo nuevoN = new Nodo(valor,ultimo.sgte);
			ultimo.sgte = nuevoN;
			ultimo = nuevoN; 
		}
		nDatos++;
	}
	
	/**
	 * Metodo para obtener el primer elemento de la cola circular sin alterar esta.
	 * @return Object -- elemento al frente de la cola circular.
	 */
	public Object tomar() {
		if(estaVacia())
			return null;
		else
			return ultimo.sgte.elemento;
	}
	
	/**
	 * Metodo para eliminar el primer elemento de la cola circular.
	 */
	public void eliminar() {
		if(!estaVacia()) {
			if(nDatos == 1)
				ultimo = null;
			else
				ultimo.sgte = ultimo.sgte.sgte;
			
			nDatos--;
		}
	}
	
	/**
	 * Metodo para obtener el ultimo elemento de la cola circular sin alterar esta.
	 * @return Object -- elemento al final de la cola circular.
	 */
	public Object ultimo() {
		if(ultimo != null)
			return ultimo.elemento;
		else
			return null;
	}
	
	/**
	 * Metodo para rotar la cola circular. 
	 */
	public void rotar() {
		if(!estaVacia())
			ultimo = ultimo.sgte;
	}
	
	/**
	 * Metodo para obtener un iterador sobre la cola circular.
	 * @return Iterator -- iterador sobre la cola circualr.
	 */
	public java.util.Iterator iterador() {
		return new MiIterador();
	}
	
	private class MiIterador implements java.util.Iterator {
		private Nodo posicion = ultimo;
		private int tmpNdatos = nDatos;
		
		public boolean hasNext() {
			return (posicion != null && tmpNdatos>0);
		}
		
		public Object next() {
			if(hasNext()){
				Object o = posicion.sgte.elemento;
				posicion = posicion.sgte;
				tmpNdatos--;
				return o;
			}
			throw new NoSuchElementException("No hay elementos en la Cola Circular.");
		}
		
		// Metodo no soportado
		public void remove() {
			throw new UnsupportedOperationException();
		}
	}

}